package com.gx.obe.step.price.calculator;

import java.math.BigDecimal;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

import com.gx.obe.components.core.util.MoneyUtils;
import com.gx.obe.step.price.beans.PriceDetail;
import com.gx.obe.step.price.beans.PriceDetailEvalResult;
import com.gx.obe.step.price.beans.PriceResult;
import com.gx.obe.step.price.beans.SupplierPriceDetail;
import com.gx.obe.step.price.beans.SupplierPriceDetailEvalResult;
import com.gx.obe.util.utils.UuidUtils;

/**
 * @Description: 价格明细评审结果计算器
 * @author chenxw
 */
public class PriceDetailEvalResultCalculator {
	
	/**
	 * @Description: 项目id
	 */
	private final String tenderId;
	
	/**
	 * @Description: 价格公式计算器
	 */
	private final PriceFormulaCalculator calculator;
	
	/**
	 * @author chenxw
	 * @param tenderId
	 * @param calculator
	 */
	public PriceDetailEvalResultCalculator(String tenderId, PriceFormulaCalculator calculator) {
		Objects.requireNonNull(tenderId);
		Objects.requireNonNull(calculator);
		
		this.tenderId = tenderId;
		this.calculator = calculator;
	}
	
	/**
	 * @Description: 计算价格明细评审结果
	 * @author chenxw
	 * @param priceDetail
	 * @return
	 */
	public PriceDetailEvalResult calculate(PriceDetail priceDetail) {
		Objects.requireNonNull(priceDetail);
		
		BigDecimal weight = Optional.ofNullable(priceDetail.getScoreWeight()).map(BigDecimal::new).map(BigDecimal.valueOf(0.01)::multiply).orElse(BigDecimal.ZERO);
		
		String detailCode = priceDetail.getDetailCode();
		List<SupplierPriceDetail> spdList = priceDetail.getSupplierPriceDetailList();
		calculator.init(getPriceList(spdList));
		PriceDetailEvalResult pder = new PriceDetailEvalResult();
		pder.setId(UuidUtils.getUUID());
		pder.setTenderId(tenderId);
		pder.setDetailCode(detailCode);
		pder.setBasePrice(calculator.getBasePrice());
		pder.setSupplierPriceDetailEvalResultList(spdList.stream().map(t -> createEvalResult(detailCode, weight, t)).collect(Collectors.toList()));
		return pder;
	}
	
	/**
	 * @Description: 获取化价格列表
	 * @author chenxw
	 * @param spdList
	 * @return
	 */
	private List<BigDecimal> getPriceList(List<SupplierPriceDetail> spdList) {
		return spdList.stream().map(SupplierPriceDetail::getEvalPrice).map(this::getMoneyFormat).collect(Collectors.toList());
	}
	
	/**
	 * @Description: 生成供应商价格明细评审结果
	 * @author chenxw
	 * @param detailCode
	 * @param priceDetail
	 * @param weight
	 * @return
	 */
	private SupplierPriceDetailEvalResult createEvalResult(String detailCode, BigDecimal weight, SupplierPriceDetail priceDetail) {
		SupplierPriceDetailEvalResult spder = new SupplierPriceDetailEvalResult();
		spder.setId(UuidUtils.getUUID());
		spder.setTenderId(tenderId);
		spder.setDetailCode(detailCode);
		spder.setSupplierId(priceDetail.getSupplierId());
		PriceResult priceResult = calculator.calculate(getMoneyFormat(priceDetail.getEvalPrice()));
		spder.setDeviationRate(priceResult.getDeviationRate());
		spder.setScore(priceResult.getScore());
		spder.setWeight(weight);
		return spder;
	}
	
	private BigDecimal getMoneyFormat(String moneyStr) {
		return MoneyUtils.getMoneyFromText(moneyStr, calculator.getPriceUnit());
	}
	
}
