package com.gx.obe.step.price.composite;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.step.price.beans.PriceDetail;
import com.gx.obe.step.price.beans.PriceDetailEvalResult;
import com.gx.obe.step.price.beans.SupplierPriceDetailEvalResult;
import com.gx.obe.step.price.calculator.PriceDetailEvalResultCalculator;
import com.gx.obe.step.price.calculator.PriceFormulaCalculator;
import com.gx.obe.step.price.calculator.SupplierPriceEvalResultCalculator;
import com.gx.obe.step.price.service.PriceDetailEvalResultService;
import com.gx.obe.step.price.service.PriceDetailService;
import com.gx.obe.step.price.table.SupplierTreeTable;
import com.gx.obe.util.utils.CollectionUtils;
import com.gx.obe.web.entity.EvaluationFactorResult;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.evaluation.EvaluationFactor;
import com.gx.obe.web.entity.evaluation.EvaluationStep;
import com.gx.obe.web.entity.tender.TenderProjectRuleEntity;
import com.gx.obe.web.service.SupplierService;
import com.gx.obe.web.service.evaluation.EvaluationFactorResultService;

import de.kupzog.ktable.SWTX;

/**
 * @Description: 报价明细评审界面
 * @author chenxw
 */
public class PriceDetailComposite extends Composite {
	
	private final SupplierService supplierService = new SupplierService();
	private final EvaluationFactorResultService evaluationFactorResultService = new EvaluationFactorResultService();
	
	/**
	 * @Description: 项目id
	 */
	private String tenderId;
	/**
	 * @Description: 用户id
	 */
	private String userId;
	/**
	 * @Description: 项目规则
	 */
	private TenderProjectRuleEntity projectRule;
	/**
	 * @Description: 评审步骤
	 */
	private EvaluationStep evaluationStep;
	/**
	 * @Description: 评审指标
	 */
	private EvaluationFactor evaluationFactor;
	/**
	 * @Description: 供应商列表
	 */
	private List<Supplier> supplierList;
	/**
	 * @Description: 价格公式计算器
	 */
	private PriceFormulaCalculator calculator;
	
	/**
	 * @Description: 价格明细列表
	 */
	private List<PriceDetail> priceDetailList;
	/**
	 * @Description: 价格明细评审结果列表
	 */
	private List<PriceDetailEvalResult> evalResultList;
	/**
	 * @Description: 价格指标评审结果列表（明细汇总结果）
	 */
	private List<EvaluationFactorResult> factorResultList;
	
	/**
	 * @Description: 表格界面
	 */
	private Composite tableComposite;
	
	/**
	 * @author chenxw
	 * @param parent
	 * @param style
	 */
	public PriceDetailComposite(Composite parent, int style) {
		super(parent, style);
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		setLayout(gridLayout);
		
		tableComposite = new Composite(this, SWT.NONE);
		tableComposite.setLayout(new FillLayout(SWT.HORIZONTAL));
		tableComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
	}
	
	/**
	 * @Description: 刷新
	 * @author chenxw
	 * @param tenderId
	 * @param supplierList
	 * @param calculator
	 */
	public void refresh(String tenderId, String userId, TenderProjectRuleEntity projectRule, EvaluationStep evaluationStep, EvaluationFactor evaluationFactor) {
		this.tenderId = tenderId;
		this.userId = userId;
		this.projectRule = projectRule;
		this.evaluationStep = evaluationStep;
		this.evaluationFactor = evaluationFactor;
		
		this.supplierList = supplierService.getEvaluationStepSupplierList(tenderId, evaluationStep);
		
		refreshPriceDetailList();
		refreshEvalResultList();
		refreshTable();
	}
	
	/**
	 * @Description: 设置计算器
	 * @author chenxw
	 * @param calculator 
	 */
	public void setCalculator(PriceFormulaCalculator calculator) {
		this.calculator = calculator;
	}
	
	/**
	 * @Description: 保存评审结果方法
	 * @author chenxw
	 * @return 
	 */
	public boolean saveEvaluationResult() {
		return PriceDetailEvalResultService.saveEvaluationResult(tenderId, evaluationStep.getId(), evaluationFactor.getId(), userId, evalResultList, factorResultList);
	}
	
	/**
	 * @Description: 刷新报价明细列表
	 * @author chenxw
	 */
	private void refreshPriceDetailList() {
		if (Objects.isNull(evaluationFactor)) {
			return;
		}
		priceDetailList = PriceDetailService.getPriceDetailList(tenderId, evaluationFactor.getBidPriceCode());
	}
	
	/**
	 * @Description: 刷新报价明细评审结果列表
	 * @author chenxw
	 */
	private void refreshEvalResultList() {
		if (CollectionUtils.isNull(supplierList) || CollectionUtils.isNull(priceDetailList)) {
			return;
		}
		// 如果当前步骤已经完成评审评审结果数据就从数据库里查询
		if (CommonEnum.COMPLETE.equals(evaluationStep.getStepStatus())) {
			evalResultList = PriceDetailEvalResultService.getPriceDetailEvalResultList(tenderId);
			factorResultList = evaluationFactorResultService.getEvaluationFactorResultList(tenderId, evaluationStep.getId(), evaluationFactor.getId(), userId);
			return;
		}
		if (Objects.isNull(calculator)) {
			return;
		}
		// 获取价格明细评审结果计算器
		PriceDetailEvalResultCalculator priceDetailEvalResultCalculator = new PriceDetailEvalResultCalculator(tenderId, calculator);
		// 计算价格明细评审结果
		evalResultList = priceDetailList.stream().map(priceDetailEvalResultCalculator::calculate).collect(Collectors.toList());
		// 计算价格指标评审结果（明细汇总结果）
		factorResultList = supplierList.stream().map(Supplier::getId).map(createEvaluationFactorResult()).collect(Collectors.toList());
	}
	
	/**
	 * @Description: 生成价格指标评审结果（明细汇总结果）
	 * @author chenxw
	 * @return 
	 */
	private Function<String, EvaluationFactorResult> createEvaluationFactorResult() {
		// 获取供应商价格评审结果计算器
		SupplierPriceEvalResultCalculator supplierPriceEvalResultCalculator = new SupplierPriceEvalResultCalculator(tenderId, evaluationStep.getId(), userId, evaluationFactor);
		
		// 以供应商为维度分组
		Map<String, List<SupplierPriceDetailEvalResult>> supplierPriceDetailEvalResultListMap = evalResultList.stream()
				.map(PriceDetailEvalResult::getSupplierPriceDetailEvalResultList)
				.map(List::stream).flatMap(Function.identity())
				.collect(Collectors.groupingBy(SupplierPriceDetailEvalResult::getSupplierId));
		
		return t -> supplierPriceEvalResultCalculator.calculate(t, supplierPriceDetailEvalResultListMap.get(t));
	}
	
	/**
	 * @Description: 刷新价格明细评审表格
	 * @author chenxw
	 */
	private void refreshTable() {
		Arrays.stream(tableComposite.getChildren()).forEach(Control::dispose);
		new SupplierTreeTable(tableComposite, SWTX.AUTO_SCROLL | SWT.FULL_SELECTION | SWTX.EDIT_ON_KEY | SWTX.FILL_WITH_LASTCOL, projectRule, supplierList, priceDetailList, evalResultList, factorResultList);
	}
	
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

}
