package com.gx.obe.step.price.table;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.gx.obe.datastruct.tree.table.ITree;
import com.gx.obe.datastruct.tree.table.StringTree;
import com.gx.obe.step.price.beans.PriceDetail;
import com.gx.obe.step.price.beans.PriceDetailEvalResult;
import com.gx.obe.step.price.beans.SupplierPriceDetail;
import com.gx.obe.step.price.beans.SupplierPriceDetailEvalResult;
import com.gx.obe.util.utils.PredicateUtils;

/** 
 * @Description: 报价明细树
 * @author chenxw 
 */
public class PriceDetailTree implements ITree {
	
	/**
	 * @Description: 报价明细
	 */
	private PriceDetail priceDetail;
	/**
	 * @Description: 报价明细评审结果
	 */
	private PriceDetailEvalResult priceDetailEvalResult;
	
	/**
	 * @Description: 基准价
	 */
	private String basePrice;
	
	/**
	 * @Description: 评标价缓存区
	 */
	private Map<Object, String> evalPriceMap = new HashMap<>();
	/**
	 * @Description: 得分缓存区
	 */
	private Map<Object, String> scoreMap = new HashMap<>();
	
	/**
	 * @Description: 子树列表
	 */
	private final List<StringTree> stringTrees = Stream.of(
			new StringTree(this, "报价", this::getEvalPrice), 
			new StringTree(this, "基准价", this::getBasePrice), 
			new StringTree(this, "得分", this::getScore)
			).collect(Collectors.toList());
	
	
	/** 
	 * @author chenxw
	 * @param priceDetail
	 * @param priceDetailEvalResult 
	 */
	public PriceDetailTree(PriceDetail priceDetail, PriceDetailEvalResult priceDetailEvalResult) {
		this.priceDetail = priceDetail;
		this.priceDetailEvalResult = priceDetailEvalResult;
		
		this.basePrice = Optional.ofNullable(priceDetailEvalResult)
				.map(PriceDetailEvalResult::getBasePrice)
				.map(String::valueOf)
				.orElse("");
	}
	
	/**
	 * @Description: 获取评标价
	 * @author chenxw
	 * @param supplierId
	 * @return 
	 */
	private String getEvalPrice(Object supplierId) {
		return evalPriceMap.computeIfAbsent(supplierId, this::createEvalPrice);
	}
	
	/**
	 * @Description: 获取得分
	 * @author chenxw
	 * @param supplierId
	 * @return 
	 */
	private String getScore(Object supplierId) {
		return scoreMap.computeIfAbsent(supplierId, this::createScore);
	}
	
	/**
	 * @Description: 获取基准价
	 * @author chenxw
	 * @param supplierId
	 * @return 
	 */
	private String getBasePrice(Object supplierId) {
		return basePrice;
	}
	
	/**
	 * @Description: 生成评标价
	 * @author chenxw
	 * @param supplierId
	 * @return 
	 */
	private String createEvalPrice(Object supplierId) {
		if (Objects.isNull(priceDetail)) return "";
		return priceDetail.getSupplierPriceDetailList().stream()
				.filter(PredicateUtils.map(SupplierPriceDetail::getSupplierId, Predicate.isEqual(supplierId)))
				.findAny()
				.map(SupplierPriceDetail::getEvalPrice)
				.map(String::valueOf)
				.orElse("");
	}
	
	/**
	 * @Description: 生成得分
	 * @author chenxw
	 * @param supplierId
	 * @return 
	 */
	private String createScore(Object supplierId) {
		if (Objects.isNull(priceDetailEvalResult)) return "";
		return priceDetailEvalResult.getSupplierPriceDetailEvalResultList().stream()
				.filter(PredicateUtils.map(SupplierPriceDetailEvalResult::getSupplierId, Predicate.isEqual(supplierId)))
				.findAny()
				.map(SupplierPriceDetailEvalResult::getScore)
				.map(String::valueOf)
				.orElse("");
	}
	
	@Override
	public ITree getRoot() {
		return null;
	}
	
	@Override
	public List<? extends ITree> getITrees() {
		return stringTrees;
	}
	
	@Override
	public String getName() {
		return priceDetail.getDetailCode();
	}
	
	@Override
	public String getKey() {
		return priceDetail.getDetailCode();
	}
	
	@Override
	public String getValue(Object key) {
		return null;
	}

}
