package com.gx.obe.step.price.table;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.gx.obe.datastruct.tree.table.ITree;
import com.gx.obe.datastruct.tree.table.StringTree;
import com.gx.obe.step.price.beans.PriceDetail;
import com.gx.obe.step.price.beans.PriceDetailEvalResult;
import com.gx.obe.step.price.beans.SupplierPriceDetail;
import com.gx.obe.step.price.beans.SupplierPriceDetailEvalResult;
import com.gx.obe.util.utils.BigDecimalUtils;
import com.gx.obe.util.utils.PredicateUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.tender.TenderProjectRuleEntity;

/** 
 * @Description: 供应商树
 * @author chenxw 
 */
public class SupplierTree implements ITree {
	
	private TenderProjectRuleEntity projectRule;
	
	private Supplier supplier;
	
	private Map<String, List<SupplierPriceDetail>> supplierPriceDetailListMap;
	private Map<String, List<SupplierPriceDetailEvalResult>> supplierPriceDetailEvalResultListMap;
	
	/**
	 * @Description: 评标价缓存区
	 */
	private Map<Object, String> evalPriceMap = new HashMap<>();
	/**
	 * @Description: 得分缓存区
	 */
	private Map<Object, String> scoreMap = new HashMap<>();
	
	/**
	 * @Description: 子树列表
	 */
	private final List<StringTree> stringTrees;

	public SupplierTree(TenderProjectRuleEntity projectRule, Supplier supplier, List<PriceDetail> priceDetailList, List<PriceDetailEvalResult> evalResultList) {
		Objects.requireNonNull(projectRule);
		
		this.projectRule = projectRule;
		this.supplier = supplier;
		
		this.supplierPriceDetailListMap = priceDetailList.stream().collect(Collectors.toMap(PriceDetail::getDetailCode, PriceDetail::getSupplierPriceDetailList));
		this.supplierPriceDetailEvalResultListMap = evalResultList.stream().collect(Collectors.toMap(PriceDetailEvalResult::getDetailCode, PriceDetailEvalResult::getSupplierPriceDetailEvalResultList));
		
		this.stringTrees = initStringTrees();
	}
	
	private List<StringTree> initStringTrees() {
		return Stream.of(
				new StringTree(this, "报价（" + projectRule.getPriceUnit() + "）", this::getEvalPrice), 
				new StringTree(this, "得分", this::getScore)
				).collect(Collectors.toList());
	}

	/**
	 * @Description: 获取评标价
	 * @author chenxw
	 * @param detailCode
	 * @return 
	 */
	private String getEvalPrice(Object detailCode) {
		return evalPriceMap.computeIfAbsent(detailCode, this::createEvalPrice);
	}
	
	/**
	 * @Description: 获取得分
	 * @author chenxw
	 * @param detailCode
	 * @return 
	 */
	private String getScore(Object detailCode) {
		return scoreMap.computeIfAbsent(detailCode, this::createScore);
	}
	
	/**
	 * @Description: 生成评标价
	 * @author chenxw
	 * @param detailCode
	 * @return 
	 */
	private String createEvalPrice(Object detailCode) {
		return supplierPriceDetailListMap.get(detailCode).stream()
				.filter(PredicateUtils.map(SupplierPriceDetail::getSupplierId, Predicate.isEqual(supplier.getId())))
				.findAny()
				.map(SupplierPriceDetail::getEvalPrice)
				.orElse("");
	}
	
	/**
	 * @Description: 生成得分
	 * @author chenxw
	 * @param detailCode
	 * @return 
	 */
	private String createScore(Object detailCode) {
		if (!supplierPriceDetailEvalResultListMap.containsKey(detailCode)) {
			return "";
		}
		return supplierPriceDetailEvalResultListMap.get(detailCode).stream()
				.filter(PredicateUtils.map(SupplierPriceDetailEvalResult::getSupplierId, Predicate.isEqual(supplier.getId())))
				.findAny()
				.map(SupplierPriceDetailEvalResult::getScore)
				.map(this::formatScore)
				.orElse("");
	}
	
	private String formatScore(BigDecimal score) {
		score = BigDecimalUtils.round(score, projectRule.getScoreRound());
		if (Objects.isNull(score)) {
			return null;
		}
		return score.toString();
	}
	
	@Override
	public ITree getRoot() {
		return null;
	}
	
	@Override
	public List<? extends ITree> getITrees() {
		return stringTrees;
	}
	
	@Override
	public String getName() {
		return supplier.getSupplierName();
	}
	
	@Override
	public String getKey() {
		return supplier.getId();
	}
	
	@Override
	public String getValue(Object key) {
		return null;
	}
	
}
