package com.gx.obe.step.price.table;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Collectors;

import org.eclipse.swt.widgets.Composite;

import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.components.core.util.MoneyUtils;
import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.datastruct.tree.table.ParseTrees;
import com.gx.obe.datastruct.tree.table.TreeSpan;
import com.gx.obe.step.price.beans.PriceDetail;
import com.gx.obe.step.price.beans.PriceDetailEvalResult;
import com.gx.obe.util.utils.BigDecimalUtils;
import com.gx.obe.util.utils.ListSortUtils;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.web.entity.EvaluationFactorResult;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.tender.TenderProjectRuleEntity;

import de.kupzog.ktable.KTable;
import de.kupzog.ktable.SWTX;
import de.kupzog.ktable.model.CommonKtableCModel;

/**
 * @Description: 以供应商为多表头的报价明细表
 * @author chenxw
 */
public class SupplierTreeTable extends KTable {
	
	private static final String SHOW_DETAIL_CODE = "code";
	private static final String SHOW_DETAIL_NAME = "name";
	
	/**
	 * @Description: 表格模型
	 */
	private CommonKtableCModel model;
	
	/**
	 * @Description: 项目规则
	 */
	private TenderProjectRuleEntity projectRule;
	
	/**
	 * @Description: 报价明细列表
	 */
	private List<PriceDetail> priceDetailList;
	/**
	 * @Description: 报价明细评审列表
	 */
	private List<PriceDetailEvalResult> evalResultList;
	/**
	 * @Description: 价格指标评审结果列表
	 */
	private List<EvaluationFactorResult> factorResultList;
	
	/**
	 * @Description: 解析树
	 */
	private final ParseTrees parseTrees;
	/**
	 * @Description: 普通表格列
	 */
	private final List<TableCol> tableColList = new ArrayList<>();
	/**
	 * @Description: 普通表格行
	 */
	private final List<TableRow> tableRowList = new ArrayList<>();
	
	/**
	 * @author chenxw
	 * @param parent
	 * @param style
	 * @param supplierList
	 * @param priceDetailList
	 * @param evalResultList
	 * @param factorResultList
	 */
	public SupplierTreeTable(Composite parent, int style, TenderProjectRuleEntity projectRule, List<Supplier> supplierList, List<PriceDetail> priceDetailList, List<PriceDetailEvalResult> evalResultList, List<EvaluationFactorResult> factorResultList) {
		super(parent, style);
		
		Objects.requireNonNull(projectRule);
		
		this.projectRule = projectRule;
		
		this.priceDetailList = Optional.ofNullable(priceDetailList).orElseGet(ArrayList::new);;
		this.evalResultList = Optional.ofNullable(evalResultList).orElseGet(ArrayList::new);
		this.factorResultList = Optional.ofNullable(factorResultList).orElseGet(ArrayList::new);
		
		this.parseTrees = createParseTrees(supplierList);
		
		initTableRowCol();
		initCommonKtableCModel();
	}
	
	/**
	 * @Description: 生成解析树
	 * @author chenxw
	 * @param supplierList
	 * @return
	 */
	private ParseTrees createParseTrees(List<Supplier> supplierList) {
		supplierList = Optional.ofNullable(supplierList).orElseGet(ArrayList::new);
		return new ParseTrees(supplierList.stream().map(t -> new SupplierTree(projectRule, t, priceDetailList, evalResultList)).collect(Collectors.toList()));
	}
	
	/**
	 * @Description: 初始化普通表格行和列
	 * @author chenxw
	 */
	private void initTableRowCol() {
		tableColList.add(new TableCol("序号", i -> String.valueOf(i + 1)));
		
		String type = PropertiesUtils.getProperty(ConfigConstants.SHOW_PRICE_DETAIL_COL_CONTENT_TYPE);
		Function<Integer, PriceDetail> getPriceDetailByIndex = priceDetailList::get;
		tableColList.add(new TableCol("报价明细", getPriceDetailByIndex.andThen(priceDetailColContentFun(type)).andThen(ObjectUtils::getObjString)));
		
		Function<String, BigDecimal> basePriceFun = evalResultList.stream().collect(Collectors.toMap(PriceDetailEvalResult::getDetailCode, PriceDetailEvalResult::getBasePrice))::get;
		tableColList.add(new TableCol("基准价（" + projectRule.getPriceUnit() + "）", getPriceDetailByIndex.andThen(PriceDetail::getDetailCode).andThen(basePriceFun).andThen(this::formatPrice)));
		
		if (PropertiesUtils.getProperty(ConfigConstants.SUPPLIER_TREE_TABLE_SHOW_WEIGHT, CommonEnum.NO).equals(CommonEnum.YES))
			tableColList.add(new TableCol("权重（%）", getPriceDetailByIndex.andThen(PriceDetail::getScoreWeight).andThen(ObjectUtils::getObjString)));
		
		Function<Object, EvaluationFactorResult> evaluationFactorResultFun = factorResultList.stream().collect(Collectors.toMap(EvaluationFactorResult::getSupplierId, Function.identity()))::get;
		tableRowList.add(new TableRow("总分", evaluationFactorResultFun.andThen(this::getEvaluationScore).andThen(this::formatScore)));
		
		ListSortUtils<EvaluationFactorResult> listSortUtils = new ListSortUtils<>(factorResultList, Comparator.comparing(EvaluationFactorResult::getEvaluationScore).reversed());
		Function<Object, Integer> scoreSortNumberFun = factorResultList.stream().collect(Collectors.toMap(EvaluationFactorResult::getSupplierId, listSortUtils::getSortNo))::get;
		tableRowList.add(new TableRow("排序", scoreSortNumberFun.andThen(this::formatSortNo)));
	}
	
	/**
	 * @Description: 报价明细列展示内容
	 * @author chenxw
	 * @return 
	 */
	private Function<PriceDetail, String> priceDetailColContentFun(String type) {
		switch (type) {
			case SHOW_DETAIL_CODE: return PriceDetail::getDetailCode;
			case SHOW_DETAIL_NAME: return PriceDetail::getName;
			default:
				return PriceDetail::getDetailCode;
		}
	}
	
	/**
	 * @Description: 格式化金额
	 * @author chenxw
	 * @param price
	 * @return 
	 */
	private String formatPrice(BigDecimal price) {
		return MoneyUtils.getMoneyFormatByUnit(price, projectRule.getPriceUnit());
	}
	
	/**
	 * @Description: 格式化得分
	 * @author chenxw
	 * @param score
	 * @return 
	 */
	private String formatScore(BigDecimal score) {
		score = BigDecimalUtils.round(score, projectRule.getScoreRound());
		if (Objects.isNull(score)) {
			return "";
		}
		return score.toString();
	}
	
	/**
	 * @Description: 格式化排序编号
	 * @author chenxw
	 * @param sortNo
	 * @return 
	 */
	private String formatSortNo(Integer sortNo) {
		return Optional.ofNullable(sortNo).map(String::valueOf).orElse("");
	}
	
	/**
	 * @Description: 根据评审指标结果获取评审得分
	 * @author chenxw
	 * @param evaluationFactorResult
	 * @return 
	 */
	public BigDecimal getEvaluationScore(EvaluationFactorResult evaluationFactorResult) {
		return Optional.ofNullable(evaluationFactorResult).map(EvaluationFactorResult::getEvaluationScore).orElse(null);
	}
	
	/**
	 * @Description: 初始化表格模型
	 * @author chenxw
	 */
	private void initCommonKtableCModel() {
		model = new CommonKtableCModel(this, parseTrees.getHeight(), tableColList.size(), priceDetailList.size() + tableRowList.size(), parseTrees.getLeafCount());
		
		for (int i = 0; i < tableColList.size(); i++) {
			model.addRowSpan(0, i, parseTrees.getHeight());
		}
		
		for (TreeSpan treeSpan : parseTrees.getWideSpanList()) {
			model.addColSpan(treeSpan.getLevel(), model.getFixedColumnCount() + treeSpan.getLeafIndex(), treeSpan.getSpanCount());
		}
		for (int i = 0; i < tableRowList.size(); i++) {
			model.addColSpan(parseTrees.getHeight() + priceDetailList.size() + i, 0, model.getFixedColumnCount());
			for (TreeSpan treeSpan : parseTrees.getWideSpanList()) {
				if (treeSpan.getLevel() == 0) {
					model.addColSpan(parseTrees.getHeight() + priceDetailList.size() + i, model.getFixedColumnCount() + treeSpan.getLeafIndex(), treeSpan.getSpanCount());
				}
			}
		}
		
		for (int i = 0; i < tableColList.size(); i++) {
			model.setColumnWidth(i, 100);
		}
		for (int i = 0; i < parseTrees.getLeafCount(); i++) {
			model.setColumnWidth(model.getFixedColumnCount() + i, 100);
		}
		
		model.setRowHeightRender(row -> 30);
		model.setCellContentRender(this::receiveContent);
		model.setCellAlignmentRender(this::receiveCellAlignment);
		
		this.setModel(model);
		this.getParent().layout();
	}
	
	/**
	 * @Description: 加载表格内容
	 * @author chenxw
	 * @param row
	 * @param col
	 * @return
	 */
	private String receiveContent(int row, int col) {
		if (row < model.getFixedRowCount()) {
			if (col < tableColList.size()) {
				return tableColList.get(col).getName();
			}
			return parseTrees.getITree(row, col - tableColList.size()).getName();
		}
		row -= model.getFixedRowCount();
		if (row < priceDetailList.size()) {
			String detailCode = priceDetailList.get(row).getDetailCode();
			if (col < tableColList.size()) {
				return tableColList.get(col).getValueFun().apply(row);
			}
			return parseTrees.get(col - tableColList.size()).getValue(detailCode);
		}
		row -= priceDetailList.size();
		if (col < tableColList.size()) {
			return tableRowList.get(row).getName();
		}
		col -= tableColList.size();
		return tableRowList.get(row).getValueFun().apply(parseTrees.get(col).getRoot().getKey());
	}
	
	public int receiveCellAlignment(int row, int col) {
		return SWTX.ALIGN_VERTICAL_CENTER | SWTX.ALIGN_HORIZONTAL_CENTER;
	}
	
}
